<?php

declare(strict_types=1);

namespace Drupal\Tests\menu_cache\Functional;

use Drupal\language\Entity\ConfigurableLanguage;
use Drupal\menu_link_content\Entity\MenuLinkContent;
use Drupal\node\Entity\Node;
use Drupal\node\Entity\NodeType;
use Drupal\system\Entity\Menu;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\content_translation\Traits\ContentTranslationTestTrait;

/**
 * Add, edit, and delete a node with menu link.
 *
 * @group menu_ui
 */
class MenuCacheTest extends BrowserTestBase {

  /**
   * An editor user.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $editor;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'menu_ui',
    'test_page_test',
    'node',
    'block',
    'locale',
    'language',
    'menu_cache',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->drupalPlaceBlock('system_menu_block:main');
    $this->drupalPlaceBlock('page_title_block');

    $this->drupalCreateContentType(['type' => 'page', 'name' => 'Basic page']);

    $this->editor = $this->drupalCreateUser([
      'access administration pages',
      'administer content types',
      'administer menu',
      'create page content',
      'administer nodes',
      'edit any page content',
      'delete any page content',
      'view own unpublished content',
    ]);
    $this->drupalLogin($this->editor);
  }

  /**
   * Tests creating, editing, deleting menu links via node form widget.
   */
  public function testSavingNode(): void {
    $node_title = 'Test page';
    $menu_link_title = 'Test menu link';

    // Create a node with a menu link.
    $edit = [
      'title[0][value]' => $node_title,
      'menu[enabled]' => 1,
      'menu[title]' => $menu_link_title,
    ];
    $this->drupalGet('node/add/page');
    $this->submitForm($edit, 'Save');
    $this->assertSession()->pageTextContains('Basic page Test page has been created.');

    // Visit front page, ensure the menu link but not the node is displayed
    // and as an authenticated user without bypass permission, the node cache
    // tag is displayed.
    $this->drupalGet('');
    $this->assertSession()->pageTextNotContains($node_title);
    $this->assertSession()->pageTextContains($menu_link_title);

    $node = $this->drupalGetNodeByTitle($node_title);
    $node_cache_tag = 'node:' . $node->id();
    $this->assertSession()->responseHeaderContains('X-Drupal-Cache-Tags', $node_cache_tag);

    // Log out, verify the front page does not contain the node cache tag.
    $this->drupalLogout();
    $this->assertSession()->pageTextNotContains($node_title);
    $this->drupalGet('');
    $this->assertSession()->responseHeaderEquals('X-Drupal-Cache', 'MISS');
    $this->drupalGet('');
    $this->assertSession()->responseHeaderEquals('X-Drupal-Cache', 'HIT');
    $this->assertSession()->responseHeaderNotContains('X-Drupal-Cache-Tags', $node_cache_tag);
    $this->assertSession()->pageTextNotContains($node_title);
    $this->assertSession()->pageTextContains($menu_link_title);

    // Log in again, save the node without changing the publication status
    // or the menu title.
    $this->drupalLogin($this->editor);
    $this->drupalGet($node->toUrl('edit-form'));
    $this->submitForm([], 'Save');
    $this->assertSession()->pageTextContains('Basic page Test page has been updated.');

    $this->drupalGet('');
    $this->assertSession()->responseHeaderContains('X-Drupal-Cache-Tags', $node_cache_tag);

    // Log out again, the front page should still be a cache hit.
    $this->drupalLogout();
    $this->assertSession()->pageTextNotContains($node_title);
    $this->drupalGet('');
    $this->assertSession()->responseHeaderEquals('X-Drupal-Cache', 'HIT');
    $this->assertSession()->responseHeaderNotContains('X-Drupal-Cache-Tags', $node_cache_tag);
    $this->assertSession()->pageTextNotContains($node_title);
    $this->assertSession()->pageTextContains($menu_link_title);

    // Log in again, change the menu title.
    $updated_menu_link_title = 'Updated menu link title';
    $this->drupalLogin($this->editor);
    $this->drupalGet($node->toUrl('edit-form'));
    $this->submitForm(['menu[title]' => $updated_menu_link_title], 'Save');
    $this->assertSession()->pageTextContains('Basic page Test page has been updated.');

    // Menu link is still visible for the editor.
    $this->drupalGet('');
    $this->assertSession()->responseHeaderContains('X-Drupal-Cache-Tags', $node_cache_tag);
    $this->assertSession()->pageTextNotContains($node_title);
    $this->assertSession()->pageTextNotContains($menu_link_title);
    $this->assertSession()->pageTextContains($updated_menu_link_title);

    // Log out again, the front page should be a cache MiSS and the link title
    // should be updated.
    $this->drupalLogout();
    $this->assertSession()->pageTextNotContains($node_title);
    $this->drupalGet('');
    $this->assertSession()->responseHeaderEquals('X-Drupal-Cache', 'MISS');
    $this->drupalGet('');
    $this->assertSession()->responseHeaderEquals('X-Drupal-Cache', 'HIT');
    $this->assertSession()->responseHeaderNotContains('X-Drupal-Cache-Tags', $node_cache_tag);
    $this->assertSession()->pageTextNotContains($node_title);
    $this->assertSession()->pageTextNotContains($menu_link_title);
    $this->assertSession()->pageTextContains($updated_menu_link_title);

    // Log in again, unpublish the node.
    $this->drupalLogin($this->editor);
    $this->drupalGet($node->toUrl('edit-form'));
    $this->submitForm(['status[value]' => '0'], 'Save');
    $this->assertSession()->pageTextContains('Basic page Test page has been updated.');

    // Menu link is still visible for the editor.
    $this->drupalGet('');
    $this->assertSession()->responseHeaderContains('X-Drupal-Cache-Tags', $node_cache_tag);
    $this->assertSession()->pageTextNotContains($node_title);
    $this->assertSession()->pageTextContains($updated_menu_link_title);

    // Log out again, the front page should be a cache MiSS and the link
    // should not be displayed.
    $this->drupalLogout();
    $this->assertSession()->pageTextNotContains($node_title);
    $this->drupalGet('');
    $this->assertSession()->responseHeaderEquals('X-Drupal-Cache', 'MISS');
    $this->drupalGet('');
    $this->assertSession()->responseHeaderEquals('X-Drupal-Cache', 'HIT');
    $this->assertSession()->responseHeaderNotContains('X-Drupal-Cache-Tags', $node_cache_tag);
    $this->assertSession()->pageTextNotContains($node_title);
    $this->assertSession()->pageTextNotContains($menu_link_title);
    $this->assertSession()->pageTextNotContains($updated_menu_link_title);


  }

}
